RandomCompanionOptions = {};

local function debugmsg(msg)
	if RandomCompanion_settings.DEBUG then
		DEFAULT_CHAT_FRAME:AddMessage('RandomCompanion Debug: ' .. msg);
	end
end

function RandomCompanionOptions.Initialize()
	if RandomCompanionOptions.Initialized ~= true then
		debugmsg('Initializing RandomCompanionOptions');
		
		--Move the Mount/Summon button over so we have room for the slider
		CompanionSummonButton:SetPoint('TOPLEFT', 45, -265);
		
		RandomCompanionOptions.WeightSlider = CreateFrame('Slider', nil, PetPaperDollFrameCompanionFrame, 'OptionsSliderTemplate');
		RandomCompanionOptions.WeightSlider:SetWidth(100);
		RandomCompanionOptions.WeightSlider:SetHeight(20);
		RandomCompanionOptions.WeightSlider:SetOrientation('HORIZONTAL');
		RandomCompanionOptions.WeightSlider:SetPoint('TOPLEFT', 220, -273);
		RandomCompanionOptions.WeightSlider:SetMinMaxValues(0, 50);
		RandomCompanionOptions.WeightSlider.tooltipText = 'RandomCompanion Weight\n\n|cFFFFFFFFSet the value higher to make RandomCompanion use this companion more often.';
		RandomCompanionOptions.WeightSlider:SetValueStep(1);
		RandomCompanionOptions.WeightSlider:EnableMouseWheel(true);
		RandomCompanionOptions.WeightSlider:Show();
		
		local text = RandomCompanionOptions.WeightSlider:CreateFontString(nil, 'BACKGROUND');
		text:SetFontObject('GameFontHighlightSmall');
		text:SetPoint('LEFT', RandomCompanionOptions.WeightSlider, 'TOPLEFT', 0, 3);
		RandomCompanionOptions.WeightSlider.valText = text;
		
		RandomCompanionOptions.WeightSlider:SetScript('OnMouseWheel', function (self, arg1)
			local step = self:GetValueStep() * arg1
			local value = self:GetValue()
			local minVal, maxVal = self:GetMinMaxValues()
		
			if step > 0 then
				self:SetValue(min(value+step, maxVal))
			else
				self:SetValue(max(value+step, minVal))
			end
		end);
		
		RandomCompanionOptions.WeightSlider:SetScript("OnValueChanged", function(self, value)
			local selected = PetPaperDollFrame_FindCompanionIndex();
			
			if (selected > 0) then
				local creatureID, creatureName = GetCompanionInfo(PetPaperDollFrameCompanionFrame.mode, selected);
				value = RandomCompanionOptions.WeightSlider:GetValue();
				if (value == 0) then
					RandomCompanionOptions.WeightSlider.valText:SetText('Weight: Disabled');
				else
					RandomCompanionOptions.WeightSlider.valText:SetText('Weight: ' .. value);
				end
				
			end
			
			--Wait a little while before re-initializing because it can lockup WoW for a second while moving the slider
			RandomCompanionOptions.SaveAndReloadTime = GetTime() + 0.25;
			RandomCompanionOptions.EventFrame:SetScript("OnUpdate", RandomCompanionOptions.SaveAndReload);
		end);
		
		RandomCompanionOptions.Initialized = true;
	end
end

function RandomCompanionOptions.SaveAndReload()
	if (GetTime() > RandomCompanionOptions.SaveAndReloadTime) then
		RandomCompanionOptions.EventFrame:SetScript("OnUpdate", nil);
		
		local selected = PetPaperDollFrame_FindCompanionIndex();
		
		if (selected > 0) then
			local creatureID, creatureName = GetCompanionInfo(PetPaperDollFrameCompanionFrame.mode, selected);
			value = RandomCompanionOptions.WeightSlider:GetValue();
			RandomCompanion.SetWeight(creatureName, value);
			RandomCompanion.Initialize(false);
		end
	end
end

local origPetPaperDollFrame_UpdateCompanionPreview = PetPaperDollFrame_UpdateCompanionPreview;
function PetPaperDollFrame_UpdateCompanionPreview(...)
	local selected = PetPaperDollFrame_FindCompanionIndex();
	
	if (selected > 0) then
		local creatureID, creatureName = GetCompanionInfo(PetPaperDollFrameCompanionFrame.mode, selected);
		RandomCompanionOptions.WeightSlider:SetValue(RandomCompanion.GetWeight(creatureName));
		--Remove the automatic save and reload because it's not necessary
		RandomCompanionOptions.EventFrame:SetScript("OnUpdate", nil);
	end
	
	return origPetPaperDollFrame_UpdateCompanionPreview(...);
end

function RandomCompanionOptions.Event(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		RandomCompanionOptions.Initialize();
	end
end

RandomCompanionOptions.EventFrame = CreateFrame("Frame", "RandomCompanionOptionsFrame");
RandomCompanionOptions.EventFrame:SetScript("OnEvent", RandomCompanionOptions.Event);
RandomCompanionOptions.EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
